#include "html.h"

#include <ESP8266WiFi.h>
#include "ESP8266WebServer.h"
#include "ESP8266HTTPClient.h"

// OTA
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>

#include <DallasTemperature.h>
#include <OneWire.h>

OneWire oneWire(D1);
DallasTemperature sensors(&oneWire);

ESP8266WebServer server;

WiFiClient client;

HTTPClient http_a;
  
int connecting = 0;
int a = 1;
int b = 1;
int relay1 = D5; 
int relay2 = D3; 
int relay3 = D7; 
int relay1_state = 0; 
int relay2_state = 0; 
int relay3_state = 0; 
int stoper = 0;
int tempTime = 0;
unsigned long currentTime = 0;

int trybAuto = 1;

int temp_hi_water = 55;
int temp_low_water = 45;
int temp_hi_air = 24;
int temp_low_air = 18;

int termostat = 1;
int termostatDirection = 0;

float h_val; 
float t_val;
float w_val = 0;

const String SSID = "********"; // wifi network name
const String PASS = "********"; // password for access to wifi network

#include <Adafruit_Sensor.h>
#include "DHTesp.h"
DHTesp dht;

#include <LiquidCrystal_I2C.h>

//LiquidCrystal_I2C lcd(0x27, 16, 2);
LiquidCrystal_I2C lcd(0x3f, 16, 2);

void heaterGetStatus()
{
  String txt = "";
  txt = (String(txt) + t_val);
  
  String stat = "{\"tempAir\":\"" + txt + "\" , \"tempWater\": \"" + w_val + "\", \"relay1\": " + relay1_state + ", \"relay2\": " + relay2_state + ", \"relay3\": " + relay3_state + ", \"termostat\": " + termostat + ", \"trybAuto\": " + trybAuto + ", \"tempTermostat\": " + temp_hi_air + " }";
  
  server.send(200,"application/json",stat);
}

void setWebRelay1(int stat){
  setRelay1(stat);
  server.send(200,"text/plain","ok");
}

void setRelay1(int stat){
  relay1_state = stat;
  digitalWrite(relay1,relay1_state);
}

void setWebRelay2(int stat){
  setRelay2(stat);
  server.send(200,"text/plain","ok");
}

void setRelay2(int stat){
  relay2_state = stat;
  digitalWrite(relay2,relay2_state);
}

void setWebRelay3(int stat){
  setRelay3(stat);
  server.send(200,"text/plain","ok");
}

void setRelay3(int stat){
  relay3_state = stat;
  digitalWrite(relay3,relay3_state);
}

void setWebTermostatTemp(){
  temp_hi_air = server.arg("temp").toInt();
  lcd.setCursor(14, 1);
  lcd.print(temp_hi_air);
  server.send(200,"text/plain","ok");
}

void setTermostat(){
  termostat = server.arg("val").toInt();
  server.send(200,"text/plain","ok");
}

void setTrybAuto(){
  trybAuto = server.arg("val").toInt();
  server.send(200,"text/plain","ok");
}

void setup() {

  pinMode(relay1,OUTPUT);
  pinMode(relay2,OUTPUT);
  pinMode(relay3,OUTPUT);
  setRelay1(0);
  setRelay2(0);
  setRelay3(1);

  //OTA
  WiFi.mode(WIFI_STA);

  lcd.init();
  lcd.begin(16,2);
  lcd.backlight();
  
  lcd.setCursor(0, 0);
  lcd.print("WIFI Starting ...");
  lcd.setCursor(0, 1);
  
  WiFi.begin(SSID,PASS);

  IPAddress staleIP(192, 168, 1, 52);
  IPAddress gateway(192, 168, 1, 254); 
  IPAddress subnet(255, 255, 255, 0);
  IPAddress dns(8, 8, 8, 8);

  WiFi.config(staleIP, gateway, subnet, dns);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    lcd.print(".");
  }
  
  WiFi.setAutoReconnect(true);
  WiFi.persistent(true);
  
  lcd.setCursor(0, 0);
  lcd.print("                ");
  lcd.setCursor(0, 1);
  lcd.print("                ");
  lcd.setCursor(0, 1);
  lcd.print(WiFi.localIP());

  lcd.setCursor(13, 1);
  lcd.print("T");
  lcd.setCursor(14, 1);
  lcd.print(temp_hi_air);
  
  server.on("/",[](){server.send(200,"text/html",INDEX_HTML);});
  server.on("/getstatus",heaterGetStatus);

  server.on("/relay1on",[](){setWebRelay1(1);});
  server.on("/relay1off",[](){setWebRelay1(0);});
  server.on("/relay2on",[](){setWebRelay2(1);});
  server.on("/relay2off",[](){setWebRelay2(0);});
  server.on("/relay3on",[](){setWebRelay3(1);});
  server.on("/relay3off",[](){setWebRelay3(0);});
  server.on("/termostattemp",setWebTermostatTemp);
  server.on("/termostat",setTermostat);
  server.on("/trybauto",setTrybAuto);

  server.enableCORS(true);

  server.begin();

  dht.setup(D4, DHTesp::DHT22);

  server.handleClient();

  Serial.begin(115200);
  sensors.begin();

  // OTA
  ArduinoOTA.onStart([]() {
    String type;
    if (ArduinoOTA.getCommand() == U_FLASH) {
      type = "sketch";
    } else { // U_FS
      type = "filesystem";
    }

    // NOTE: if updating FS this would be the place to unmount FS using FS.end()
    Serial.println("Start updating " + type);
  });
  ArduinoOTA.onEnd([]() {
    Serial.println("\nEnd");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    Serial.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) {
      Serial.println("Auth Failed");
    } else if (error == OTA_BEGIN_ERROR) {
      Serial.println("Begin Failed");
    } else if (error == OTA_CONNECT_ERROR) {
      Serial.println("Connect Failed");
    } else if (error == OTA_RECEIVE_ERROR) {
      Serial.println("Receive Failed");
    } else if (error == OTA_END_ERROR) {
      Serial.println("End Failed");
    }
  });
  ArduinoOTA.begin();
  
  setRelay1(1);
  setRelay2(0);
  setRelay3(1);
}

void loop() {
  ArduinoOTA.handle();

  currentTime = millis();
  server.handleClient();

  if (currentTime - tempTime > 3000) {
    tempTime = currentTime;

    t_val = dht.getTemperature();

    sensors.requestTemperatures();
    w_val = sensors.getTempCByIndex(0);
  }

  if (currentTime - stoper > 1000) {
    stoper = currentTime;

    lcd.setCursor(0, 0);
    lcd.print("Air: ");
    lcd.print(t_val);
    lcd.print(" C");
    lcd.setCursor(0, 1);
    lcd.print("Water: ");
    lcd.print(w_val);
    lcd.print(" C");

    if (trybAuto==1) {
      if (w_val>temp_hi_water) setRelay2(0);
      if (w_val<temp_low_water) setRelay2(1);
    }

    if (termostat==1) {
      if (t_val>temp_hi_air) {
        if (termostatDirection!=2) {
          trybAuto=0;
          setRelay1(0);
          setRelay2(0);
          setRelay3(1);
          delay(60000);
          setRelay3(0);
          termostatDirection = 2; // opada
        }
      }
      if (t_val<(temp_hi_air-2)) {
        if (termostatDirection!=1) {
          trybAuto=1;
          setRelay1(1);
          setRelay2(1);
          setRelay3(1);
          termostatDirection = 1; // rośnie
        }
      }
    }
    a++;
  }

  if (w_val>65) {
    setRelay1(0);
    setRelay2(0);
    trybAuto=0;
  }
} 

